{ CSV Ver2.2

  区切り文字による項目抽出コンポーネント


  (C)1996-2013 千々岩幸治

  1999.4.29  Ver2.0 　・区切り文字が連続するとトークンが抽出できない
  　　　　　　      　       　バグを修正(これが出来ないと意味無し...)。
  　　　　　　　　　　         ・わかりにくかった処理を見直し。
                     ・このコンポーネントで不必要だと思われる機能
                      　の削除。（文字抜き取りとファイル関連機能）
  2001.2.12  Ver2.1 　・文字列処理を行った時、抽出した文字の頭と終わりに
                       付く文字列区切り文字を削除するメソッドを追加。
                       （Ver2.0でこれに相当する機能を削除していた）
  2013.2.20  Ver2.2  ・Lazarus用に修正。
}

unit csvu;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs;
//  Classes, SysUtils;

type
  csv = class(TComponent)
  private
    { Private declarations }
    ptext:string; {対象文字列格納エリア}
    ptokns:string;  {区切り文字（列）}
    pstokns:string; {ペア内を無視する時に指定する区切り文字（列）}
    pstrflg:boolean;  {指定文字のペアの内の区切り文字は無視}
    pblockflg:boolean;  {区切り文字の連続をまとまりで見る}
  protected
    { Protected declarations }
  public
    { Public declarations }
    pitems:tstringlist; {抽出された項目が格納されるエリア}

    constructor create(aowner:tcomponent); override;
    destructor destroy; override;

    procedure strtok;
    procedure delstrflg;

  published
    { Published declarations }
    property text:string read ptext write ptext;
    property tokns:string read ptokns write ptokns;
    property stokns:string read pstokns write pstokns;
    property strflg:boolean read pstrflg write pstrflg;
    property items:tstringlist read pitems write pitems;
    property blockflg:boolean read pblockflg write pblockflg;
 end;

procedure Register;

implementation

constructor csv.create(aowner:tcomponent);
begin
  inherited create(aowner); {親のコンストラクタを呼ぶ}
  items:=tstringlist.create;
  tokns:=','; {デフォルトのトークン}
  stokns:='"';  {デフォルトの文字列認識トークン}
  strflg:=true; {文字列処理を行う}
  blockflg:=false;  {区切り文字を個別に見る}
end;

destructor csv.destroy;
begin
  items.free;
  inherited destroy;  {親のデストラクタを呼ぶ}
end;

{トークン抽出}
procedure csv.strtok;
var
  kaishi:integer;
  owari:integer;
  moji:string;
  work:string;
  max:integer;
  flag:boolean;
begin
  kaishi:=1;  {抽出中の項目の先頭}
  owari:=1; {抽出中の項目の終わり}

  flag:=false;  {非分割処理は初めは開始していない}

  max:=length(text);

  {項目消去}
  items.clear;

  {対象文字列が何もなかったら何もしない}
  if text='' then
    exit;

  while owari<=max do
  begin
    moji:=copy(text,owari,1); {1文字抜き取る}

    {非分割文字の処理}
    if pos(moji,stokns)<>0 then {非分割区切り文字}
    begin
      if strflg then
      begin
        if flag then  {非分割処理中}
          flag:=false {非分割処理終了}
        else
          flag:=true; {非分割処理開始}
      end;
      inc(owari);

      continue;
    end;

    {区切り文字の処理}
    if pos(moji,tokns)<>0 then  {区切り文字}
    begin
      if strflg and flag then {非分割処理中}
        {何もしない}
      else
      begin
        work:=copy(text,kaishi,owari-kaishi);
        items.add(work);

        if blockflg then
        begin
          {区切り文字が連続している場合，ひと固まりで取り扱う用に
          次の区切り文字までパスする}
          inc(owari);
          while owari<=max do
          begin
            moji:=copy(text,owari,1); {1文字抜き取る}
            if pos(moji,tokns)<>0 then  {区切り文字}
              inc(owari)
            else
            begin
              dec(owari);
              break;
            end;
          end;
        end;
        kaishi:=owari+1;  {開始位置を最後の項目の次に設定}
        if owari>=max then
          items.add('');  {トークンで終了していれば空データを登録}
      end;
    end;
    inc(owari);
  end;

  {最後の項目の登録処理}
  if kaishi<>owari then
  begin
    work:=copy(text,kaishi,owari-kaishi);
    items.add(work);
  end;
end;

//文字列処理の頭とお尻の区切り文字を削除
procedure csv.delstrflg;
var
  counter:integer;
  moji:string;
  mojiretu:string;
begin
  for counter:=1 to pitems.Count do
  begin
    //頭の文字が区切り文字なら削除
    mojiretu:=pitems[counter-1];
    moji:=copy(mojiretu,1,1);
    if pos(moji,pstokns)<>0 then
      mojiretu:=copy(mojiretu,2,length(mojiretu)-1);

    //お尻の文字が区切り文字なら削除
    moji:=copy(mojiretu,length(mojiretu),1);
    if pos(moji,pstokns)<>0 then
      mojiretu:=copy(mojiretu,1,length(mojiretu)-1);

    pitems[counter-1]:='';
    pitems[counter-1]:=mojiretu;
  end;
end;

procedure Register;
begin
  {$I csv_icon.lrs}
  RegisterComponents('mycomponent',[csv]);
end;

end.
